@@ -3,14 +3,11 @@ Copyright 2021 The Microsoft DeepSpeed Team
 '''
 import torch
 import os
-from datetime import timedelta
 from torch.nn.modules import Module
 import torch.distributed as dist
 from ..runtime.state_dict_factory import SDLoaderFactory
 from ..runtime.weight_quantizer import WeightQuantization
 from ..module_inject.replace_module import replace_transformer_layer
-from ..constants import INFERENCE_GENERIC_MODE
-from ..module_inject import replace_policy
 from ..utils import logger, init_distributed
 
 from ..pipe import PipelineModule
@@ -29,8 +26,23 @@ class InferenceEngine(Module):
                  return_tuple=True,
                  replace_method='auto',
                  quantization_setting=None):
+        """
+        Args:
+            model: torch.nn.Module
+            mp_size: model-parallel size
+            mpu: model-parallel unit (used for Megatron-type models)
+            checkpoint: the json-path, showing the address of model-checkpoints
+                Example: {type: 'Megatron', 'checkpoints': [ckpt_mp0.pt, ckpt_mp1.pt], 'version': 1.0}
+            dtype: data-type by which inference is executed
+            injection_dict: the dictionary that shows the injection policy:
+                Example: {BertLayer: HFBertLayerPolicy}
+            return_tuple: if true, inference-API returns a tuple, otherwise a tensor
+            replace_method: the injection method, this can be passed as auto if no injection-policy is defined, in which case the injection is automatic based on the available policies
+            quantization_setting:
+                one of None, Tuple(mlp_extra_grouping, quantize_groups), quantize_groups
+        """
 
-        super(InferenceEngine, self).__init__()
+        super().__init__()
 
         self.module = model
 
@@ -47,7 +59,7 @@ class InferenceEngine(Module):
         self.quantize_merge_count = 1
         self.quantization_scales = None
 
-        self._check_quantize_setting(quantization_setting)
+        self._init_quantization_setting(quantization_setting)
 
         if self.checkpoint:
             self._load_checkpoint(self.checkpoint)
@@ -66,11 +78,15 @@ class InferenceEngine(Module):
         # apply injection policy
         if self.injection_dict:
             for client_module, injection_policy in self.injection_dict.items():
-                self._apply_injection_policy(client_module, injection_policy)
-        elif replace_method == 'auto':
+                self._apply_injection_policy(client_module,
+                                             injection_policy,
+                                             return_tuple)
+        elif replace_method == "auto":
             self._apply_injection_policy()
 
-        self.module.to(torch.cuda.current_device())
+        device = torch.cuda.current_device()
+        logger.info(f"Place model to device: {device}")
+        self.module.to(device)
 
         if self.mp_world_size > 1:
             self.model_orig_fwd = self.module.forward
@@ -79,14 +95,8 @@ class InferenceEngine(Module):
             self.module.register_forward_pre_hook(self._pre_forward_hook)
 
     def _get_model_config_generate(self):
-        if hasattr(self.module, 'config'):
-            self.config = self.module.config
-        else:
-            self.config = None
-        if hasattr(self.module, 'generate'):
-            self.generate = self.module.generate
-        else:
-            self.generate = None
+        self.config = getattr(self.module, 'config', None)
+        self.generate = getattr(self.module, 'generate', None)
 
     def _create_model_parallel_group(self):
         # Call the init process
@@ -102,38 +112,47 @@ class InferenceEngine(Module):
         else:
             self.mp_group = InferenceEngine.inference_mp_group
 
-    def _check_quantize_setting(self, quantization_setting):
+    def _init_quantization_setting(self, quantization_setting):
         self.quantize_bits = 8
         self.mlp_extra_grouping = False
         self.quantize_groups = 1
-        if quantization_setting is None:
-            return
-        elif type(quantization_setting) is tuple:
+        if type(quantization_setting) is tuple:
             self.mlp_extra_grouping, \
             self.quantize_groups = quantization_setting
-        else:
+        elif quantization_setting is not None:
             self.quantize_groups = quantization_setting
+        logger.info(f"quantize_bits = {self.quantize_bits} "
+                    f"mlp_extra_grouping = {self.mlp_extra_grouping}, "
+                    f"quantize_groups = {self.quantize_groups}")
 
     def _validate_args(self, mpu):
-        assert isinstance(self.module, Module)
-        assert isinstance(self.mp_world_size, int)
-        assert self.mp_world_size >= 1
+        if not isinstance(self.module, Module):
+            raise ValueError(f"model must be a torch.nn.Module, got {type(self.module)}")
+        if not isinstance(self.mp_world_size, int) or self.mp_world_size < 1:
+            raise ValueError(f"mp_size must be an int >= 1, got {self.mp_world_size}")
 
         if mpu:
-            methods = [f"get_model_parallel_group"]
-            methods.extend([f"get_data_parallel_group"])
+            methods = ["get_model_parallel_group", "get_data_parallel_group"]
             for method in methods:
-                assert hasattr(mpu, method), f"mpu is missing {method}"
+                if not hasattr(mpu, method):
+                    raise ValueError(f"mpu is missing {method}")
+        if self.checkpoint is not None and not isinstance(self.checkpoint, str):
+            raise ValueError(
+                f"checkpoint must be None or a str, got {type(self.checkpoint)}")
 
-        assert self.checkpoint is None or isinstance(self.checkpoint, str)
+        supported_dtypes = [None, torch.half, torch.int8, torch.float]
+        if self.dtype not in supported_dtypes:
+            raise ValueError(
+                f"{self.dtype} not supported, valid dtype: {supported_dtypes}")
 
-        supported_dtypes = [torch.half, torch.int8, torch.float]
-        assert self.dtype is None or self.dtype in supported_dtypes, f"dtype={self.dtype} is not in the \
-            list of supported dtypes {supported_dtypes}"
+        if self.injection_dict is not None and not isinstance(self.injection_dict, dict):
+            raise ValueError(
+                f"injection_dict must be None or a dict, got: {self.injection_dict}")
 
-        assert self.injection_dict is None or isinstance(self.injection_dict, dict)
-
-    def _apply_injection_policy(self, client_module=None, injection_policy=None):
+    def _apply_injection_policy(self,
+                                client_module=None,
+                                injection_policy=None,
+                                return_tuple=True):
         replace_transformer_layer(client_module,
                                   self.module,
                                   policy=injection_policy,
@@ -153,8 +172,9 @@ class InferenceEngine(Module):
         sd_loader = SDLoaderFactory.get_sd_loader_json(load_dir)
         is_pipe_parallel = isinstance(self.module, PipelineModule)
 
-        assert (not is_pipe_parallel),\
-        'pipeline parallelism is currently not supported in inference.'
+        if is_pipe_parallel:
+            raise RuntimeError(
+                'pipeline parallelism is currently not supported in inference.')
 
         mp_rank = 0 if self.mp_group is None else dist.get_rank(group=self.mp_group)
 
